#include "SimpleGL.h"

int SimpleGL::orbit = 0;
int SimpleGL::rotate = 0;
int SimpleGL::drift = 0;

void SimpleGL::initGL()
{
  glClearColor( 0.0, 0.0, 0.0, 0.0 );
  glShadeModel( GL_SMOOTH );
  glEnable( GL_DEPTH_TEST );

  // lighting
  glEnable(GL_LIGHTING);
  GLfloat light0_position[] = {-10.0, 10.0, 10.0, 1.0};
  GLfloat light0_specular[] = {0.0, 1.0, 0.0, 1.0};
  glLightfv(GL_LIGHT0, GL_POSITION, light0_position);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light0_specular);
  glEnable(GL_LIGHT0);
}

void SimpleGL::display()
{    
  glClear( GL_COLOR_BUFFER_BIT );
  glClear( GL_DEPTH_BUFFER_BIT );

  // draw the center sphere
  glPushMatrix();
    GLfloat surface_one[] = { 1.0, 1.0, 0.0, 0.0 };
    glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, surface_one );
    glutSolidSphere( 1.0, 32, 24 );
  glPopMatrix();
  
  // draw the satellite sphere
  glPushMatrix();
    glRotatef( (GLfloat) drift, 0.0, 0.0, 1.0 );
    glRotatef( (GLfloat) orbit, 0.0, 1.0, 0.0 );
    glTranslatef( 2.0, 0.0, 0.0 );
    glRotatef( (GLfloat) rotate, 0.0, 1.0, 0.0 );
    GLfloat surface_two[] = { 0.0, 1.0, 0.0, 0.0 };
    glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, surface_two );
    glutSolidSphere( 0.4, 5, 4 );
  glPopMatrix();

  glutSwapBuffers();
}

void SimpleGL::reshape (int w, int h)
{
  // setup the camera view with perspective
  glViewport( 0, 0, (GLsizei) w, (GLsizei) h );
  glMatrixMode( GL_PROJECTION );
  glLoadIdentity();
  gluPerspective( 60.0, (GLfloat) w/(GLfloat) h, 1.0, 20.0 );
  glMatrixMode( GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt( 0.0, 0.0, 5.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0 );
}

void SimpleGL::animate( int value )
{
  // rotate the satellite
  rotate = (rotate + 20) % 360;

  // move satellite around
  orbit = (orbit + 5) % 360;

  // satellite drift
  drift = (drift + 1) % 360;
  glutPostRedisplay();

  // glutTimer is one shot.  reset timer.
  glutTimerFunc (REFRESH_MS, animate, value);
}

SimpleGL::SimpleGL( int argc, char** argv )
{
  glutInit( &argc, argv );
  glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB );
  glutInitWindowSize( 500, 500 ); 
  glutInitWindowPosition( 100, 100 );
  glutCreateWindow( argv[0] );

  this->initGL ();

  glutDisplayFunc( display );
  glutReshapeFunc( reshape );

  glutTimerFunc( REFRESH_MS, animate, 0 );
}

void SimpleGL::mainLoop()
{
  glutMainLoop();
}
